@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: Rbind.c,v 1.115 2001/12/12 14:42:06 karls Exp $";
+"$Id: Rbind.c,v 1.118 2005/01/24 10:24:21 karls Exp $";
 
 int
 Rbind(s, name, namelen)
@@ -301,8 +301,8 @@ Rbind(s, name, namelen)
 
 	/* did we get the requested port? */
 	/* LINTED pointer casts may be troublesome */
-	if (TOIN(name)->sin_port != htons(0)
-	&& TOIN(name)->sin_port != TOIN(&socksfd.remote)->sin_port) { /* no. */
+	if (TOCIN(name)->sin_port != htons(0)
+	&& TOCIN(name)->sin_port != TOIN(&socksfd.remote)->sin_port) { /* no. */
 		/*
 		 * Since the socket is already bound locally, "unbind" it so caller
 		 * doesn't get confused.
