@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: authneg.c,v 1.52 2001/12/12 14:42:09 karls Exp $";
+"$Id: authneg.c,v 1.54 2005/05/28 17:13:24 michaels Exp $";
 
 int
 negotiate_method(s, packet)
@@ -79,8 +79,11 @@ negotiate_method(s, packet)
 		return -1;
 
 	/* read servers response for method it wants to use */
-	if (readn(s, response, sizeof(response), &packet->auth) != sizeof(response))
+	if ((rc = readn(s, response, sizeof(response), &packet->auth))
+	!= sizeof(response)){
+		swarn("%s: readn(), %d out of %d", function,rc, sizeof(response));
 		return -1;
+	}
 
 	if (request[AUTH_VERSION] != response[AUTH_VERSION]) {
 		swarnx("%s: got replyversion %d, expected %d",
