@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: udp.c,v 1.129 2005/01/24 10:24:22 karls Exp $";
+"$Id: udp.c,v 1.131 2005/05/28 17:13:25 michaels Exp $";
 
 /* ARGSUSED */
 ssize_t
@@ -88,7 +88,8 @@ Rsendto(s, msg, len, flags, to, tolen)
 			slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)",
 			function, protocol2string(SOCKS_TCP),
 			sockaddr2string(&socksfd->local, dststring, sizeof(dststring)),
-			sockaddr2string(&socksfd->server, srcstring, sizeof(srcstring)), n);
+			sockaddr2string(&socksfd->server, srcstring, sizeof(srcstring)),
+			(unsigned long)n);
 
 			return n;
 		}
@@ -112,7 +113,8 @@ Rsendto(s, msg, len, flags, to, tolen)
 	slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)",
 	function, protocol2string(SOCKS_TCP),
 	sockaddr2string(&socksfd->local, dststring, sizeof(dststring)),
-	sockaddr2string(&socksfd->reply, srcstring, sizeof(srcstring)), n);
+	sockaddr2string(&socksfd->reply, srcstring, sizeof(srcstring)),
+	(unsigned long)n);
 
 	return MAX(-1, n);
 }
@@ -178,7 +180,8 @@ Rrecvfrom(s, buf, len, flags, from, fromlen)
 		slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)",
 		function, protocol2string(SOCKS_TCP),
 		sockaddr2string(forus, srcstring, sizeof(srcstring)),
-		sockaddr2string(&socksfd->local, dststring, sizeof(dststring)), n);
+		sockaddr2string(&socksfd->local, dststring, sizeof(dststring)),
+		(unsigned long)n);
 
 		return n;
 	}
@@ -280,7 +283,8 @@ Rrecvfrom(s, buf, len, flags, from, fromlen)
 	slog(LOG_DEBUG, "%s: %s: %s -> %s (%lu)",
 	function, protocol2string(SOCKS_UDP),
 	sockaddr2string(&newfrom, srcstring, sizeof(srcstring)),
-	sockaddr2string(&socksfd->local, dststring, sizeof(dststring)), n);
+	sockaddr2string(&socksfd->local, dststring, sizeof(dststring)), 
+	(unsigned long)n);
 
 	if (from != NULL) {
 		*fromlen = MIN(*fromlen, newfromlen);
@@ -297,6 +301,7 @@ udpsetup(s, to, type)
 	const struct sockaddr *to;
 	int type;
 {
+	const char *function = "udpsetup()";
 	struct socks_t packet;
 	struct socksfd_t socksfd;
 	struct sockaddr_in newto;
@@ -304,6 +309,8 @@ udpsetup(s, to, type)
 	socklen_t len;
 	int p;
 
+	slog(LOG_DEBUG, "%s: s = %d", function, s);
+
 	if (!socks_addrisok((unsigned int)s))
 		socks_rmaddr((unsigned int)s);
 
