@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,7 @@
  *  Software Distribution Coordinator  or  sdc@inet.no
  *  Inferno Nettverk A/S
  *  Oslo Research Park
- *  Gaustadalllllllen 21
+ *  Gaustadallen 21
  *  NO-0349 Oslo
  *  Norway
  *
@@ -51,7 +51,7 @@
 #endif  /* HAVE_STRVIS */
 
 static const char rcsid[] =
-"$Id: util.c,v 1.128 2001/11/12 14:10:04 michaels Exp $";
+"$Id: util.c,v 1.137 2005/01/24 10:24:22 karls Exp $";
 
 /* fake "ip address", for clients without DNS access. */
 static char **ipv;
@@ -101,7 +101,7 @@ sockscode(version, code)
 					SERRX(code);
 			}
 			/* NOTREACHED */
-		
+
 		case HTTP_V1_0:
 			switch (code) {
 				case SOCKS_SUCCESS:
@@ -153,14 +153,14 @@ sockshost2sockaddr(host, addr)
 	struct sockaddr *addr;
 {
 	const char *function = "sockshost2sockaddr()";
-	uint8_t sa_len;
+	uint8_t sa_length;
 
 	bzero(addr, sizeof(*addr));
 
 	switch (host->atype) {
 		case SOCKS_ADDR_IPV4:
 			addr->sa_family = AF_INET;
-			sa_len = sizeof(struct sockaddr_in);
+			sa_length = sizeof(struct sockaddr_in);
 
 			/* LINTED pointer casts may be troublesome */
 			TOIN(addr)->sin_addr = host->addr.ipv4;
@@ -170,7 +170,7 @@ sockshost2sockaddr(host, addr)
 			struct hostent *hostent;
 
 			addr->sa_family = AF_INET;
-			sa_len = sizeof(struct sockaddr_in);
+			sa_length = sizeof(struct sockaddr_in);
 
 			if ((hostent = gethostbyname(host->addr.domain)) == NULL
 			||   hostent->h_addr_list == NULL) {
@@ -195,7 +195,7 @@ sockshost2sockaddr(host, addr)
 	}
 
 #if HAVE_SOCKADDR_SA_LEN
-	addr->sa_len = sa_len;
+	addr->sa_len = sa_length;
 #endif
 
 	/* LINTED pointer casts may be troublesome */
@@ -211,7 +211,7 @@ fakesockshost2sockaddr(host, addr)
 {
 	const char *function = "fakesockshost2sockaddr()";
 	char string[MAXSOCKSHOSTSTRING];
-	uint8_t sa_len;
+	uint8_t sa_length;
 
 #if SOCKS_CLIENT /* may be called before normal init, log to right place. */
 	clientinit();
@@ -225,7 +225,7 @@ fakesockshost2sockaddr(host, addr)
 	switch (host->atype) {
 		case SOCKS_ADDR_DOMAIN:
 			addr->sa_family = AF_INET;
-			sa_len = sizeof(struct sockaddr_in);
+			sa_length = sizeof(struct sockaddr_in);
 
 			/* LINTED pointer casts may be troublesome */
 			if (socks_getfakeip(host->addr.domain, &TOIN(addr)->sin_addr))
@@ -237,7 +237,7 @@ fakesockshost2sockaddr(host, addr)
 	}
 
 #if HAVE_SOCKADDR_SA_LEN
-	addr->sa_len = sa_len;
+	addr->sa_len = sa_length;
 #endif
 
 	/* LINTED pointer casts may be troublesome */
@@ -376,10 +376,10 @@ hostname2sockaddr(name, index, addr)
 			TOIN(addr)->sin_addr = *(struct in_addr *)hostent->h_addr_list[i];
 			/* LINTED pointer casts may be troublesome */
 			TOIN(addr)->sin_port = htons(0);
-			
+
 			return addr;
 		}
-	
+
 	return NULL;
 }
 
@@ -390,24 +390,24 @@ ifname2sockaddr(ifname, index, addr)
 	struct sockaddr *addr;
 {
 	int i;
-	struct ifaddrs ifa, *ifap = &ifa, *iface; 
+	struct ifaddrs ifa, *ifap = &ifa, *iface;
 
 	if (getifaddrs(&ifap) != 0)
 		return NULL;
-	 
+
 	for (iface = ifap, i = 0; i <= index && iface != NULL;
-	iface = iface->ifa_next) 
-		if (strcmp(iface->ifa_name, ifname) == 0 
-		&& iface->ifa_addr != NULL && iface->ifa_addr->sa_family == AF_INET) { 
+	iface = iface->ifa_next)
+		if (strcmp(iface->ifa_name, ifname) == 0
+		&& iface->ifa_addr != NULL && iface->ifa_addr->sa_family == AF_INET) {
 			if (i++ != index)
 				continue;
 
 			*addr = *iface->ifa_addr;
-			freeifaddrs(ifap); 
+			freeifaddrs(ifap);
 			return addr;
 		}
 
-	freeifaddrs(ifap); 
+	freeifaddrs(ifap);
 	return NULL;
 }
 
@@ -420,6 +420,8 @@ socks_addfakeip(host)
 	char **tmpmem;
 	struct in_addr addr;
 
+ 	/* XXX mutex_lock */
+
 	if (socks_getfakeip(host, &addr) == 1)
 		return addr.s_addr;
 
@@ -443,6 +445,8 @@ error "\"FAKEIP_END\" can't be smaller than \"FAKEIP_START\""
 
 	strcpy(ipv[ipc], host);
 
+	/* XXX mutex_unlock */
+
 	return htonl(ipc++ + FAKEIP_START);
 }
 
@@ -694,13 +698,17 @@ socketoptdup(s)
 		{ SOL_SOCKET,	SO_SNDBUF			},
 #endif
 
+#if HAVE_SO_SNDLOWAT
 #ifdef SO_RCVLOWAT
 		{ SOL_SOCKET,	SO_RCVLOWAT			},
 #endif
+#endif /* HAVE_SO_SNDLOWAT */
 
+#if HAVE_SO_SNDLOWAT
 #ifdef SO_SNDLOWAT
 		{ SOL_SOCKET,	SO_SNDLOWAT			},
 #endif
+#endif /* HAVE_SO_SNDLOWAT */
 
 #ifdef SO_RCVTIMEO
 		{ SOL_SOCKET,	SO_RCVTIMEO			},
@@ -897,7 +905,7 @@ socks_lock(descriptor, type, timeout)
 	int type;
 	int timeout;
 {
-	const char *function = "socks_lock()"; 
+	const char *function = "socks_lock()";
 	struct flock lock;
 	int rc;
 
@@ -1015,9 +1023,8 @@ int
 fdisopen(fd)
 	int fd;
 {
-	if (fcntl(fd, F_GETFD, 0) == 0)
-		return 1;
-	return 0;
+
+	return fcntl(fd, F_GETFD, 0) != -1;
 }
 
 void
@@ -1052,7 +1059,7 @@ snprintfn(str, size, format, va_alist
 #else
 	va_start(ap);
 #endif  /* STDC_HEADERS */
-	
+
 	rc = vsnprintf(str, size, format, ap);
 
 	/* LINTED expression has null effect */
@@ -1067,7 +1074,7 @@ snprintfn(str, size, format, va_alist
 }
 
 /*
- * Posted by Kien Ha (Kien_Ha@Mitel.COM) in comp.lang.c once upon a 
+ * Posted by Kien Ha (Kien_Ha@Mitel.COM) in comp.lang.c once upon a
  * time.
 */
 int
