@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd_child.c,v 1.142 2004/12/24 18:43:09 michaels Exp $";
+"$Id: sockd_child.c,v 1.144 2005/06/10 11:04:07 michaels Exp $";
 
 #define MOTHER	0	/* descriptor mother reads/writes on.	*/
 #define CHILD	1	/* descriptor child reads/writes on.	*/
@@ -452,7 +452,6 @@ childcheck(type)
 		SASSERTX((*childv)[child].freec <= max);
 		proxyc += type < 0 ? max : (*childv)[child].freec;
 
-#if 0 /* Hasn't been tested enough to be enabled. */
 		if ((*childv)[child].freec == max) {
 			++idle;
 
@@ -465,7 +464,6 @@ childcheck(type)
 				--child; /* everything was shifted once to the left. */
 			}
 		}
-#endif
 	}
 
 	if (type >= 0)
@@ -506,6 +504,8 @@ fillset(set)
 			FD_SET(sockscf.internalv[i].s, set);
 			dbits = MAX(dbits, sockscf.internalv[i].s);
 		}
+	else
+		swarn("can't accept new clients, no free negotiate slots");
 
 	/* negotiator children. */
 	for (i = 0; i < negchildc; ++i) {
@@ -515,7 +515,7 @@ fillset(set)
 			dbits = MAX(dbits, negchildv[i].s);
 		}
 
-		/* we can always accept a ack ofcourse. */
+		/* we can always accept an ack ofcourse. */
 		SASSERTX(negchildv[i].ack >= 0);
 		FD_SET(negchildv[i].ack, set);
 		dbits = MAX(dbits, negchildv[i].ack);
@@ -529,7 +529,7 @@ fillset(set)
 			dbits = MAX(dbits, reqchildv[i].s);
 		}
 
-		/* we can always accept a ack ofcourse. */
+		/* we can always accept an ack ofcourse. */
 		SASSERTX(reqchildv[i].ack >= 0);
 		FD_SET(reqchildv[i].ack, set);
 		dbits = MAX(dbits, reqchildv[i].ack);
